#<profile4xi profile = "20" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "socketclient4xi.py">
#<abstract4xi abstract = "provides the function SocketClient4Xi"/>
#<refer4xi reference = "connection.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------


# -------------------------------------------------------------------------------------------------
# Public API 
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'SocketClient4Xi' ]  
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import errno
import io
import itertools
import os
import pickle
import select
import socket
import struct
import sys
import time
import tempfile
try:
    import _winapi
    from _winapi import WAIT_OBJECT_0, WAIT_TIMEOUT, INFINITE
except ImportError:
    if sys.platform == 'win32':
        raise
    _winapi = None   
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
from multiprocessing import connection
from multiprocessing.connection import SocketClient
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                               
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from address_type4xi import address_type4xi

from print4xi import print4xi
from sleep4xi import omega2sleep4xi 
from time4xi import time4xi
#"
#/>


# -------------------------------------------------------------------------------------------------
#<function4xi function = "SocketClient4Xi" usage = "
#     connection.py
#         def Client(address, family=None, authkey=None):        
#             if family == 'AF_PIPE':
#                 c = PipeClient(address)
#             else:
#                 c = SocketClient(address)
#
#     client4xi.py:
#         def Client4Xi(exchange, family, passport):
#             connection2xi = SocketClient4Xi(exchange)
#
#"> 
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/> 
# -------------------------------------------------------------------------------------------------
def SocketClient4Xi(hub):
    '''
    Returns a connection object connected to the socket given by `hub`
    '''
    
    # -----------------------------------------------------------------------------------------    
    # (Step 1/4)
    # -----------------------------------------------------------------------------------------    
    class2print = "SocketClient4Xi"
    method2print = "SocketClient4Xi"
    
    message2print = "{(Step 1/4a) started" + "; SocketClient4Xi-started: " + str(SocketClient4Xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    
    message2print = "{(Step 1/4b) passed" + "; hub-passed-to-SocketClient4Xi: " + str(hub) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)  
          
    ack2hub = True
    if hub is None:
        message2print = "{(Step 1/4c) yielded" + "; hub-yielded-by-SocketClient4Xi: " + str(hub)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
    
        ack2hub = False
        
    # -----------------------------------------------------------------------------------------    
    # (Step 2/4)
    # -----------------------------------------------------------------------------------------        
    family2xi = address_type4xi(hub)                                                                                #dragonized; always returns a reasonable family
    message2print = "{(Step 2/4) set" + "; family2xi-set-by-SocketClient4Xi to: " + str(family2xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)  

    
    # -----------------------------------------------------------------------------------------    
    # (Step 3/4)
    # -----------------------------------------------------------------------------------------        
    ack2socket = False
    nack2socket = True
    
    socket2xi = None
    try:
        socket2xi = socket.socket( getattr(socket, family2xi) )
        
        message2print = "{(Step 3/4a) set" + "; socket2xi-set-by-SocketClient4Xi to: " + str(socket2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        
        ack2socket = True
        
        message2print = "{(Step 3/4b) set" + "; ack2socket-set-by-SocketClient4Xi to: " + str(ack2socket)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
    except:
        message2print = "{(Step 3/4c) yielded" + "; nack2socket-yielded-by-SocketClient4Xi: " + str(nack2socket)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
    
    # -----------------------------------------------------------------------------------------    
    # (Step 4/4)
    # -----------------------------------------------------------------------------------------        
    with socket2xi as s2xi:
        s2xi.setblocking(True)
        message2print = "{(Step 4/4a) performed" + "; setblocking-performed-by-SocketClient4Xi: " + str(s2xi.setblocking)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        
        message2print = "{(Step 4/4b) using" + "; hub-using-by-SocketClient4Xi: " + str(hub)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        
        s2xi.connect(hub)
        message2print = "{(Step 4/4c) performed" + "; connect-performed-by-SocketClient4Xi: " + str(s2xi.connect)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        
        message2print = "{(Step 4/4d) using" + "; detach-using-by-SocketClient4Xi: " + str(s2xi.detach)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        
        message2print = "{(Step 4/4e) returning" + "; Connection-returning-by-SocketClient4Xi: " + str(Connection)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        
        return Connection(s2xi.detach())
    
    
#</function4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of socketclient4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>

